<?php
namespace AutomateYourTests\Controller;

/**
 * This class drives all debug functions
 *
 * @link        http://automateyourtests.com
 * @author      Marc Daniel Dammann <howto@automateyourtests.com>
 * @version     1.2
 * @copyright   2016 Marc Daniel Dammann
 * @license     GPL-3.0 <https://opensource.org/licenses/GPL-3.0>
 *
 * Copyright 2016 Marc Daniel Dammann
 * This file is part of AutomateYourTests. AutomateYourTests is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version. AutomateYourTests is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with AutomateYourTests. If not, see http://www.gnu.org/licenses/.
 */

class Debugger {
    /**
     * Class vars
     */
    protected $verbose = '';
    protected $interactive = '';
    protected $from = 0;
    protected $to = 0;
    protected $debug = false;

    /**
     * Has the 'verbose' argument been passed (= verbose mode been enabled) ?
     */
    public function beVerbose($failure)
    {
        if ($failure === 1 && $this->verbose === 'fail') {
            return true;
        } elseif ($this->verbose === 'all' || $this->verbose === true) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Has the 'interactive' argument been passed (= interactive mode been enabled) ?
     */
    public function isInteractive()
    {
        return $this->interactive;
    }

    /**
     * Has the 'debug' argument been passed (= debug mode been enabled) ?
     */
    public function isDebug()
    {
        return $this->debug;
    }

    /**
     * Has the 'to' argument been set to 'fail' ?
     */
    public function terminateIfFail()
    {
        if ($this->to === 'fail') {
            return true;
        } else {
            return false;
        }
    }

    /**
     * TODO: Debug impersonation user login (step-by-step)
     */
    /* protected function stepUserLoginInteractively($userLabel, $failure)
    {
        //$this->partialLog[] = $this->view->logger->separator();

        // Release and reset log
        // echo $this->getLog();
        // $this->setLog('');

        // Release prompt
        $key = $this->readSingleCommandLineCharacter("Press SPACE to re-run last user login, ESC to exit test suite, or ANY OTHER KEY to continue ...\n");

        // If SPACE, elseif ESC key
        if ($key == chr(32)) {
            $this->controller->impersonator->resetPreviousUserLoginCookie($userLabel);
            return $this->runUserLoginStep($userLabel);
        } elseif ($key == chr(27)) {
            $this->partialLog[] = $this->view->logger->separator();
            return -1;
        } else {
            return $failure;
        }
    } */

    /**
     * Debug <step> (step-by-step)
     */
    protected function stepInteractively($fieldName, $i, $level)
    {
        // Release prompt
        $key = $this->readSingleCommandLineCharacter("Press SPACE to re-run last step, ESC to exit test suite, or ANY OTHER KEY to continue ...\n");

        // If SPACE, elseif ESC key
        if ($key == chr(32)) {
            // WORKAROUND: Because cascadeDownSingleNode() already increased the count at the first run
            //$this->view->logger->allCount--;
            //$this->view->logger->failureCount--;

            return $this->controller->cascader->cascadeDownSingleNode($fieldName, $i, $level);
        } elseif ($key == chr(27)) {
            return -1;
        }
    }

    /**
     *
     */
    public function hasFrom()
    {
        if ($this->from > 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     *
     */
    public function hasTo()
    {
        if ($this->to > 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     *
     */
    public function hasReachedFrom($xmlNode)
    {
        if ($this->from > 0) {
            $domNode = \dom_import_simplexml($xmlNode);

            if ($domNode->getLineNo() >= $this->from) {
                return true;
            } else {
                return false;
            }
        }
    }

    /**
     *
     */
    public function hasReachedTo($xmlNode)
    {
        //if ($this->to == 0) {
        //    return true;
        //} else {
        $domNode = \dom_import_simplexml($xmlNode);

        if ($domNode->getLineNo() >= $this->to) {
            return true;
        } else {
            return false;
        }
        //}
    }

    /**
     *
     */
    protected function readSingleCommandLineCharacter($promptMessage)
    {
        echo $promptMessage;

        // TODO: Protect
        readline_callback_handler_install('', function() {});

        $character = stream_get_contents(STDIN, 1);

        // TODO: Protect
        readline_callback_handler_remove();

        return $character;
    }

    /**
     *
     */
    public function offerToStepInteractively($fieldName, $i, $level, $failure)
    {
        if ($this->isInteractive()) {
            if ($failure === 1) {
                $this->view->logger->releaseLog();
                $failure = $this->stepInteractively($fieldName, $i, $level);

                $this->view->logger->resetLog();
            }
        } else {
            //$this->view->logger->addToLog();
            //$this->view->logger->resetLog();
        }

        return $failure;
    }

    /**
     * Setters
     */
    public function setVerbose($verbose)
    {
        $this->verbose = $verbose;
    }
    public function setInteractive($interactive)
    {
        $this->interactive = $interactive;
    }
    public function setFrom($from)
    {
        $this->from = $from;
    }
    public function setTo($to)
    {
        $this->to = $to;
    }
    public function setDebug($debug)
    {
        $this->debug = $debug;
    }
}
