<?php
namespace AutomateYourTests\Model;

/**
 * This class asserts for every test <step> whether its assertion
 * is true/false (means: whether the test <step> is a pass/fail).
 *
 * @link        http://automateyourtests.com
 * @author      Marc Daniel Dammann <howto@automateyourtests.com>
 * @version     1.2
 * @copyright   2016 Marc Daniel Dammann
 * @license     GPL-3.0 <https://opensource.org/licenses/GPL-3.0>
 *
 * Copyright 2016 Marc Daniel Dammann
 * This file is part of AutomateYourTests. AutomateYourTests is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version. AutomateYourTests is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with AutomateYourTests. If not, see http://www.gnu.org/licenses/.
 */

class Asserter {
    /**
     * Test that the response string contains a specified sub string
     */
    public function testContains($htmlString, $containString)
    {
        // Default
        $isFailure = true;

        // SimpleXmlObject to string
        $containString = (string) $containString;

        if ($htmlString === '' && $containString === '') {
            // Empty string comparison
            $isFailure = false;
        } elseif (strpos($htmlString, $containString) !== false) {
            // Case-sensitive comparison
            $isFailure = false;
        //} elseif (stripos($htmlString, $containString) !== false) {
        //    TODO: Case-insensitive comparison
        //    $isFailure = false;
        }

        return $isFailure;
    }

    /**
     * Test that the response string does NOT contain a specified sub string
     */
    public function testNotcontains($htmlString, $notContainString)
    {
        // Default
        $isFailure = true;

        // SimpleXmlObject to string
        $notContainString = (string) $notContainString;

        if ($htmlString === '' && $notContainString !== '') {
            // Empty string comparison
            $isFailure = false;
        } elseif ($htmlString !== '' && $notContainString === '') {
            // Empty string comparison
            $isFailure = false;
        } elseif (strpos($htmlString, $notContainString) === false) {
            // Case-sensitive comparison
            $isFailure = false;
        //} elseif (stripos($htmlString, $notContainString) === false) {
        //    TODO: Case-insensitive comparison
        //    $isFailure = false;
        }

        return $isFailure;
    }

    /**
     * Test that a specified JSON XPath expression matches
     */
    public function testXpath($jsonString, $xpathExpression)
    {
        // Default
        $isFailure = true;

        // Attempt to parse response as JSON
        if (\AutomateYourTests\Model\Generic\Strings::isJson($jsonString)) {
            $jsonArray = \json_decode($jsonString, true);

            if (is_array($jsonArray)) {
                $simpleXmlObject = \AutomateYourTests\Model\Config\ExtendedXMLElement::fromArray($jsonArray);

                // TODO: Check instanceof SimpleXMLElement?
                if ($simpleXmlObject) {
                    if ($simpleXmlObject->xpath($xpathExpression)) {
                        $isFailure = false;
                    }
                }
            }
        }

        return $isFailure;
    }

    /**
     * Test that an actual HTTP code is in the expected HTTP code array
     */
    public function testStatus($actualHttpStatus, $expectedHttpStatuses)
    {
        // Default
        $isFailure = true;

        // SimpleXmlObject to array
        $expectedHttpStatuses = (array) $expectedHttpStatuses;

        if (in_array($actualHttpStatus, $expectedHttpStatuses)) {
            $isFailure = false;
        }

        return $isFailure;
    }

    /**
     * Test that an actual HTTP code is NOT in the expected HTTP code array
     */
    public function testNotstatus($actualHttpStatus, $notExpectedHttpStatuses)
    {
        // Default
        $isFailure = true;

        // SimpleXmlObject to array
        $notExpectedHttpStatuses = (array) $notExpectedHttpStatuses;

        if (! in_array($actualHttpStatus, $notExpectedHttpStatuses)) {
            $isFailure = false;
        }

        return $isFailure;
    }
}
