<?php
namespace AutomateYourTests\Model\Config;

/**
 * This class makes sure that the test suite's XML has a predictable structure
 *
 * @link        http://automateyourtests.com
 * @author      Marc Daniel Dammann <howto@automateyourtests.com>
 * @version     1.2
 * @copyright   2016 Marc Daniel Dammann
 * @license     GPL-3.0 <https://opensource.org/licenses/GPL-3.0>
 *
 * Copyright 2016 Marc Daniel Dammann
 * This file is part of AutomateYourTests. AutomateYourTests is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version. AutomateYourTests is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with AutomateYourTests. If not, see http://www.gnu.org/licenses/.
 */

class Normaliser extends Base {
    /**
     * Class vars
     */

    // TODO: Move all this into the Validator() class
    public $baseFields;
    public $testAssertionFields;
    public $definedFields;
    public $fixtureFields;

    /**
     * Prepend each 'url' element with the 'baseurl'
     */
    public function completeUrls($levels = array())
    {
        // Every config array level must have a valid key returned
        $fieldName = $this->getStructureNodeName($levels);

        foreach ($levels->$fieldName as $level) {
            if ($level->baseurl && $level->url) {
                $level->url = $level->baseurl . $level->url;
                unset($level->baseurl);
            }

            if (! $this->isBottomLevel($level)) {
                $this->completeUrls($level);
            }
        }
    }

    /**
     * Prepend each 'url' element with the 'baseurl'
     */
    public function completeUserImpersonationUrls($config = array())
    {
        if ($config->impersonator->user) {
            foreach ($config->impersonator->user as $user) {
                if ($user->baseurl && $user->url) {
                    $user->url = $user->baseurl . $user->url;
                    unset($user->baseurl);
                }
            }
        }
    }

    /**
     *
     */
    public function removeXmlComments($config = array())
    {
        if ($config->comment instanceof \AutomateYourTests\Model\Config\ExtendedXMLElement) {
            unset($config->comment);
        }
    }

    /**
     *
     */
    public function getBaseFields()
    {
        return $this->baseFields;
    }

    /**
     *
     */
    public function getFixtureFields()
    {
        return $this->fixtureFields;
    }

    /**
     *
     */
    public function hasOnlyFixtures($level = array())
    {
        if ($level instanceof \AutomateYourTests\Model\Config\ExtendedXMLElement) {
            $fixtureFields = $this->getFixtureFields();

            // Prepare array for use with array_intersect_key()
            $fixtureFieldsKeys = array_flip($fixtureFields);

            // Array from object
            $levelArray = (array) $level;

            if (array_intersect_key($fixtureFieldsKeys, $levelArray)) {
                return true;
            }
        }

        return false;
    }

    /**
     *
     */
    public function getDefinedFields()
    {
        return $this->definedFields;
    }

    /**
     *
     */
    public function hasBaseField($level = array())
    {
        if ($level instanceof \AutomateYourTests\Model\Config\ExtendedXMLElement) {
            $baseFields = $this->getBaseFields();

            // Prepare array for use with array_intersect_key()
            $baseFieldElementKeys = array_flip($baseFields);

            // Array from object
            $levelArray = (array) $level;

            if (array_intersect_key($levelArray, $baseFieldElementKeys)) {
                return true;
            }
        }

        return false;
    }

    /**
     *
     */
    public function hasDefinedField($level = array())
    {
        if ($level instanceof \AutomateYourTests\Model\Config\ExtendedXMLElement) {
            $definedFields = $this->getDefinedFields();

            // Prepare array for use with array_intersect_key()
            $definedFieldsKeys = array_flip($definedFields);

            // Array from object
            $levelArray = (array) $level;

            if (array_intersect_key($levelArray, $definedFieldsKeys)) {
                return true;
            }
        }

        return false;
    }

    /**
     *
     */
    public function hasTestAssertion($level = array())
    {
        if ($level instanceof \AutomateYourTests\Model\Config\ExtendedXMLElement) {
            $testAssertionFields = $this->getTestAssertionFields();

            // Array from object
            $levelArray = (array) $level;

            // Prepare array for use with array_intersect_key()
            $testAssertionNameKeys = array_flip($testAssertionFields);

            if (array_intersect_key($levelArray, $testAssertionNameKeys)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Every config array level must have a key of either of one of these
     */
    public function getStructureNodeName($levels = array())
    {
        $definedFields = $this->getDefinedFields();

        foreach ((array) $definedFields as $definedFieldName) {
            if ($levels->$definedFieldName) {
                return $definedFieldName;
            }
        }

        return '';
        //throw new \Exception('The level "' . json_encode($levels) . '" does not start with a valid field name, which must be either of "' . implode(', ', $definedFields) . '"');
    }

    /**
     *
     */
    public function isBottomLevel($level = array())
    {
        // TODO: Allow bottom levels WITHOUT assertions also
        if (! $this->hasBaseField($level) && ! $this->hasDefinedField($level)) {
        //if (isset($level->url)) {
            return true;
        } else {
            return false;
        }

        //if ($this->hasDefinedField($level)) {
        //    return false;
        //} elseif (isset($level[0])) {
        //    // WORKAROUND: If there are numeric elements in $level, there still must be n-amount of user stories, scenarios, steps, ...
        //    return false;
        //}

        //return true;
    }

    /**
     *
     */
    public function getTestAssertionFields()
    {
        return $this->testAssertionFields;
    }
}
