<?php
namespace AutomateYourTests\Model;

/**
 * This class counts things like <steps> and <users>
 * and how many of them have PASSED/FAILED.
 *
 * @link        http://automateyourtests.com
 * @author      Marc Daniel Dammann <howto@automateyourtests.com>
 * @version     1.2
 * @copyright   2016 Marc Daniel Dammann
 * @license     GPL-3.0 <https://opensource.org/licenses/GPL-3.0>
 *
 * Copyright 2016 Marc Daniel Dammann
 * This file is part of AutomateYourTests. AutomateYourTests is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version. AutomateYourTests is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with AutomateYourTests. If not, see http://www.gnu.org/licenses/.
 */

class Counter {
    /**
     * Class vars
     */
    protected $count = array(
        'user' => array(
            'all' => 0,
            'pass' => 0,
            'failure' => 0
        ),
        'setup' => array(
            'all' => 0,
            'pass' => 0,
            'failure' => 0
        ),
        'teardown' => array(
            'all' => 0,
            'pass' => 0,
            'failure' => 0
        ),
    );
    protected $model = null;

    /**
     * Get set up
     */
    public function __construct($normaliser)
    {
        $this->model                     = new \stdClass();
        $this->model->config             = new \stdClass();
        $this->model->config->normaliser = $normaliser;
    }

    /**
     * How many <step> child nodes (even if deep down)
     */
    public function howManySteps($nodes, $stepCount = 0) {
        return $this->howManyNodes('step', $nodes, $stepCount);
    }

    /**
     * How many <$nodeName> child nodes (even if deep down)
     */
    protected function howManyNodes($nodeName, $nodes, $stepCount = 0)
    {
        // Get whatever the top level element is called
        $fieldName = $this->model->config->normaliser->getStructureNodeName($nodes);

        foreach ($nodes->$fieldName as $node) {
            if ($node instanceof \AutomateYourTests\Model\Config\ExtendedXMLElement) {
                if ($node->getName() == $nodeName) {
                    $stepCount++;
                } else {
                    return $this->howManyNodes($nodeName, $node->children(), $stepCount);
                }
            }
        }

        return $stepCount;
    }

    /**
     * Increase user login count for all/succeeds/fails
     */
    public function addUser($failure)
    {
        $this->add('user', $failure);
    }

    /**
     * Get user login count for all/succeeds/fails
     */
    public function howManyUsers($index)
    {
        return $this->howMany('user', $index);
    }

    /**
     * Increase <setup> count for all/succeeds/fails
     */
    public function addSetup($failure)
    {
        $this->add('setup', $failure);
    }

    /**
     * Get <setup> count for all/succeeds/fails
     */
    public function howManySetups($index)
    {
        return $this->howMany('setup', $index);
    }

    /**
     * Increase <teardown> count for all/succeeds/fails
     */
    public function addTeardown($failure)
    {
        $this->add('teardown', $failure);
    }

    /**
     * Get <teardown> count for all/succeeds/fails
     */
    public function howManyTeardowns($index)
    {
        return $this->howMany('teardown', $index);
    }

    /**
     * Increase $type count for all and succeeds/failure
     */
    protected function add($type, $failure)
    {
        $this->count[$type]['all']++;

        if ($failure === 0) {
            $this->count[$type]['pass']++;
        } elseif ($failure === 1) {
            $this->count[$type]['failure']++;
        } else {
            throw new \Exception('The PASS/FAILURE status "' . $failure . '" is not known');
        }
    }

    /**
     * Get $type count for either all/succeeds/failure
     */
    protected function howMany($type, $index)
    {
        if (isset($this->count[$type][$index])) {
            return $this->count[$type][$index];
        } else {
            throw new \Exception('The count index "' . $index . '" is not known');
        }
    }
}
