<?php
namespace AutomateYourTests\Model;

/**
 * This class executes any test fixtures (means: any <setup> and <teardown> XML tags)
 *
 * @link        http://automateyourtests.com
 * @author      Marc Daniel Dammann <howto@automateyourtests.com>
 * @version     1.2
 * @copyright   2016 Marc Daniel Dammann
 * @license     GPL-3.0 <https://opensource.org/licenses/GPL-3.0>
 *
 * Copyright 2016 Marc Daniel Dammann
 * This file is part of AutomateYourTests. AutomateYourTests is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version. AutomateYourTests is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with AutomateYourTests. If not, see http://www.gnu.org/licenses/.
 */

class Fixture {
    /**
     * Class vars
     */
    protected $config = array();

    /**
     * Execute a SETUP or TEAR DOWN task.
     * Return exec return code and string.
     * TODO: Against premature output when commands are failing, output buffering may help!?
     */
    public function executeCommand($fixtureStage = '', $label = '')
    {
        $response = array();

        $execCode = $this->getExecCommand($fixtureStage, $label);

        try {
            // This syntax populates $returnVar and $returnCode
            \exec($execCode, $returnArray, $returnCode);

            $response['commandSource'] = $execCode;
            $response['exitCode'] = $returnCode;
            $response['fileAndLine'] = '';
            $response['returnString'] = implode(' ', $returnArray);

            //if ($returnCode !== 0) {
            //    return 'Error: (' . $returnCode . ') ' . implode(' ', $returnArray);
            //}
        } catch (\Exception $e) {
            $response['commandSource'] = $execCode;
            $response['exitCode'] = $e->getCode();
            $response['fileAndLine'] = $e->getFile() . ':' . $e->getLine();
            $response['returnString'] = $e->getMessage();
        }

        return $response;
    }

    /**
     * Return some programming code that matches a specified label
     *
     * @param $fixtureStage Either 'setup' or 'teardown'
     * @param $label The label of the programming code that is to be returned
     */
    protected function getExecCommand($fixtureStage, $label)
    {
        if ($this->config->fixture->$fixtureStage) {
            foreach ($this->config->fixture->$fixtureStage as $stageFixture) {
                if ($stageFixture->attribute('label') == $label) {
                    return $stageFixture;
                }
            }
        }

        // TODO: Move this to the Validator()
        throw new \Exception('The label "' . $label . '" is not part of the config"');
    }

    /**
     * Set config class var
     */
    public function setConfig($config = array())
    {
        $this->config = $config;
    }
}
